# frozen_string_literal: true

require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::PublicHousingOffersController, type: :api do
  routes { Hermes::Engine.routes }
  include Devise::Test::ControllerHelpers

  let!(:responsible) { create(:user, roles: ['Administrator']) }
  let!(:user) { create(:user) }
  let!(:lessor) { create(:lessor) }


  describe 'POST /public_housing_offers' do
    describe 'Create housing_offer with minimal valid datas' do
      before :each do
        post(
          'create',
          data: {
            attributes: {}
          }
        )
      end
      it 'returns status code 422' do
        expect(response).to have_http_status(422)
      end
    end
  end

  describe 'GET /public_housing_offers as a guest' do
    describe 'GET /housing_offers' do
      [50, 150, 100].each_with_index do |qtt, i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            terraces_or_balconies: qtt,
            offer_status: 'LOGEMENT PUBLIE',
            lessor_id: lessor.id,
          )
        end
      end

      before :each do
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(3)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(3)
      end
    end

    describe 'GET /housing_offers filtered' do
      ['dc', 'dc', 'priority_public'].each_with_index do |channel_key, i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            housing_channel_keys: [channel_key],
            offer_status: 'LOGEMENT PUBLIE'
          )
        end
      end
      let!(:entity) {create(:entity)}
      let!(:entity_reservation) {create(:entity_reservation, housing_offer_id: housing_offer1.id, entity_id: entity.id, from: DateTime.parse("01/01/2021"), to: DateTime.parse("01/02/2021"))}

      before :each do
        housing_offer1.active_reservations.push(entity_reservation.id.to_s);
        housing_offer1.save
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(1)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(1)
      end
    end

    describe 'GET /housing_offers filtered' do
      ['dc', 'dc', 'priority_public'].each_with_index do |channel_key, i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            housing_channel_keys: [channel_key],
            offer_status: 'LOGEMENT PUBLIE'
          )
        end
      end
      let!(:entity) {create(:entity)}
      let!(:entity_reservation) {create(:entity_reservation, housing_offer_id: housing_offer1.id, entity_id: entity.id, from: DateTime.parse("01/01/2021"), to: DateTime.parse("01/02/2021"))}

      before :each do
        housing_offer1.active_reservations.push(entity_reservation.id.to_s);
        housing_offer1.save
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(1)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(1)
      end
    end

    describe 'GET /public_housing_offers with operators' do
      [
        ['accueillant', 50],
        ['bel appart', 150],
        ['Super', 150]
      ].each_with_index do |attributes, i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            offer_status: 'LOGEMENT PUBLIE',
            description: attributes[0],
            terraces_or_balconies: attributes[1]
          )
        end
      end

      let!(:housing_offer4) do
        create(
          :housing_offer,
          description: 'bel appart',
          terraces_or_balconies: 150,
          offer_status: 'NON PUBLIE'
        )
      end

      {
        1 => [
          { terraces_or_balconies: { '$lt': 100 } },
          { description: { '$in': %w[accueillant flamand] } },
          { description: { '$regex': /^S/ } }
        ],
        2 => [
          terraces_or_balconies: { '$ne': 50 }
        ]
      }.each do |qtt, parameters_setup|
        parameters_setup.each do |parameters|
          describe "with #{parameters}" do
            before :each do
              authenticated_header(request, responsible)
              get 'index', parameters
            end
            it 'returns valid offers quantity' do
              expect(response.status).to eq(200)
              expect(json['meta']['pagination']['total_objects']).to eq(qtt)
            end
          end
        end
      end
    end

    describe 'GET /public_housing_offers with dates range' do
      let(:end_dates) { [Date.new(2020, 12, 1), Date.new(2020, 8, 1), Date.new(2020, 7, 1)]}
      3.times do |i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            offer_status: "LOGEMENT PUBLIE",
            date_publication_start: Date.new(2020, 1, 2)
          )
        end
      end
      let!(:housing_offer3) do
        create(
          :housing_offer,
          offer_status: "NON PUBLIE",
          date_publication_start: Date.new(2020, 1, 2),
          publication_end_date: Date.new(2020, 8, 1)
        )
      end

      before do
        end_dates.each_with_index do |date, i|
          offer = send("housing_offer#{i}")
          offer.set(publication_end_date: date)
        end
        authenticated_header(request, responsible)
        get 'index', { date_publication_start: { '$gte': '2020-01-01' }, publication_end_date: { '$lte': '2020-08-01' } }
      end

      it 'returns valid offers quantity' do
        expect(response.status).to eq(200)
        expect(json['meta']['pagination']['total_objects']).to eq 2
      end
    end
  end

  describe 'GET /public_housing_offers/:id as a guest' do
    describe 'verif it work' do
      let!(:housing_offer1) { create(:housing_offer, description: 'accueillant', terraces_or_balconies: 50, offer_status: 'LOGEMENT PUBLIE') }
      
      it 'returns status code 200' do
        get 'show', id: housing_offer1.id
        expect(response).to have_http_status(200)
        expect(json).not_to be_empty
      end

      it 'as error when offer is reserved' do 
        housing_offer1.set(reserved: true)
        get 'show', id: housing_offer1.id
        expect(response).to have_http_status(401)
        expect(json['errors'].first).to include('Offre reservée')

      end 
    end
  end

  describe 'DELETE /public_housing_offers/:id as a guest' do
    let!(:housing_offer) { create(:housing_offer, user_id: responsible.id.to_s) }
    before :each do
      delete 'destroy',
              id: housing_offer.id.to_s
    end
    it 'returns status code 422' do
      expect(response).to have_http_status(422)
    end
  end
end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::PublicHousingOffersController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_PUBLIC_HOUSING_OFFERS'
  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:responsible) { create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator']) }
  let!(:user) { create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com') }
  let!(:applicant) { create(:applicant, user: responsible) }
  let!(:housing_request) { create(:housing_request, main_applicant_id: applicant.id.to_s) }

  context 'as guest' do
    let!(:admin) { create(:administrator) }

    path '/api/dmo/public_housing_offers' do
      get 'Retrieves all public housing_offers' do
        tags tags
        produces 'application/json'
        Hermes::Api::Dmo::HousingOffersController.acceptable_params.sort.each do |params_name, options|
          parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
        end
        paginator

        response '200', 'housing_offers found' do
          let!(:housing_offers) { create_list(:housing_offer, 5, offer_status: 'LOGEMENT PUBLIE') }

          include_context 'with integration test'
          include_context 'valid jsonapi collection' do
            let(:expected_collection) { housing_offers }
          end
        end
      end
    end

    path '/api/dmo/public_housing_offers/{id}' do
      get 'Retrieves a single housing_offer by id' do
        tags tags
        produces 'application/json'
        parameter name: :id, in: :path, type: :string, required: true
        response '200', 'housing_offer found' do
          let(:housing_offer) { create(:housing_offer, offer_status: 'LOGEMENT PUBLIE') }
          let(:id) { housing_offer.id }
          include_context 'with integration test'
          include_context 'valid jsonapi ressource' do
            let(:expected_ressource) { housing_offer }
          end
        end
      end
    end
  end
end
