# frozen_string_literal: true

require 'rails_helper'
require 'vcr_config'

dir_path = File.dirname(__FILE__)
schema_path = [dir_path, 'entrepriseDemande.xsd'].join('/')
require [Hermes::Engine.root, %w[lib hermes bo_client tasks]].flatten.join('/').to_s

RSpec.describe Hermes::BoClient::EntityRequestTask, type: :helper do
  let!(:xsd) { Nokogiri::XML::Schema(File.read(schema_path)) }
  let!(:entity) { create(:entity, siret: '00000000000001', id_rex: 'IDREX', title: 'Test entity')}
  let!(:applicant) { create(:applicant, main_employer_siret: entity.siret)}
  let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, als_id: 'AAAAAA')}
  let!(:nud) {create(:nud, unique_number: '999999999999', housing_request: hr)}

  it 'should load a valid XSD' do
    expect(xsd).to be_present
    expect(xsd.class).to be Nokogiri::XML::Schema
  end

  describe 'to_xml' do
    it 'as valid xml' do 
      so = Hermes::BoClient::EntityRequestTask.new(housing_request_id: hr.id)
      xml = so.to_xml
      expect(xsd.validate(Nokogiri::XML(xml)).length).to eq(0)
    end 

    it 'should has all values' do
      so = Hermes::BoClient::EntityRequestTask.new(housing_request_id: hr.id)
      parsed_xml = hr.parse_xml(so.to_xml)

      expect(parsed_xml["entrepriseDemande"]).to eq({
        "xmlns:ns2" => "http://v1.als.ikos.webservice.immobilier.sopra/demande/entrepriseDemande",
        "numUnique" => hr.unique_numbers.last,
        "refInterneDemande" => hr.als_id,
        "idREX" => hr.entity_id.to_s,
        "appCommerciale" => entity.title,
        "numSiret" => entity.siret
      })
    end

    it 'should has all values when not entity' do
      applicant.update(main_employer_siret: nil)
      so = Hermes::BoClient::EntityRequestTask.new(housing_request_id: hr.id)
      parsed_xml = hr.parse_xml(so.to_xml)
      
      expect(parsed_xml["entrepriseDemande"]).to eq({
        "xmlns:ns2" => "http://v1.als.ikos.webservice.immobilier.sopra/demande/entrepriseDemande",
        "numUnique" => hr.unique_numbers.last,
        "refInterneDemande" => hr.als_id,
        "idREX" => nil,
        "appCommerciale" => nil,
        "numSiret" => nil
      })
    end
  end
end
