# frozen_string_literal: true

require 'rails_helper'
require 'vcr_config'
require [Hermes::Engine.root, %w[lib hermes rex_client tasks]].flatten.join('/').to_s
require [Hermes::Engine.root, %w[lib hermes rex_client]].flatten.join('/').to_s

RSpec.describe Hermes::RexClient::UpdateStructureTask, type: :helper do
  let!(:entity) { create(:entity, title: 'MON AGENCE', id_cristal: '119057') }
  let!(:entity_parent) { create(:entity, title: 'MON AGENCE PARENT', id_cristal: '299785') }

  describe 'Existing entity' do
    it 'update entity and create parent' do
      VCR.use_cassette("rex_update_existing_entity", :match_requests_on => [:method, :uri, :path]) do
        client = Hermes::RexClient::Client.new
        rex_entity = client.search_entities({'fields.id_cristal' => entity.id_cristal}).first
        Hermes::RexClient::UpdateStructureTask.new.update_entity(rex_entity, 'Hermes::Entity', client, false)
      end
      expect(entity.reload.title).to eq "SUEZ RV SUD OUEST"
      expect(Hermes::Entity.all().count).to eq(2)
      expect(Hermes::Entity.find_by(id: entity.reload.parent_id).title).to eq("MON AGENCE PARENT")
    end
  end

  describe 'Not existing entity' do
    it 'create entity and parent' do
      VCR.use_cassette("rex_create_new_entity", :match_requests_on => [:method, :uri, :path]) do
        client = Hermes::RexClient::Client.new
        rex_entity = client.search_entities({'fields.id_cristal' => '264301'}).first
        Hermes::RexClient::UpdateStructureTask.new.update_entity(rex_entity, 'Hermes::Entity', client)
      end
      created_entity = Hermes::Entity.find_by(id_cristal: '264301')
      expect(Hermes::Entity.all().count).to eq(4)
      expect(created_entity.title).to eq "DIJON CEREALES MEUNERIE"
      expect(Hermes::Entity.find_by(id: created_entity.reload.parent_id).title).to eq("AXEREAL SCA")
    end
  end

  describe 'cre national' do
    let!(:rex_entity_json) do 
      {
        "id": "73acd55b-e64d-4de2-bc25-3ae63e70ad70",
        "idEntityDefinition": "a0c7c1dc-435a-47c8-a8ff-0ccc4396588b",
        "idRefExt": nil,
        "fields": {
          "date_fin_activite": nil,
          "grpmontantcollecte": "2489072",
          "categorie": nil,
          "entadressecodpos": "33610",
          "id_parent_rex": "",
          "liste_login": "",
          "entadressevoie3": nil,
          "siret": "70198020300726",
          "entadressevoie1": "34 Rue Antoine Primat",
          "rso": "SUEZ RV SUD OUEST",
          "sigle": "SUEZ RV SUD OUEST",
          "entadressevoie2": "CS 60257",
          "id_absorbant_cristal": "",
          "anneecollecte": "2019",
          "id_parent_cristal": "299785",
          "id_root_cristal": "299785",
          "id_root_rex": "",
          "id_cristal": "119057",
          "entmontantcollecte": "117282",
          "id_absorbant_rex": "",
          "entadressevoie": "Chez SUEZ RV CSP",
          "entadresseville": "CANEJAN",
          "app_comm": "SUEZ RV SUD OUEST",
          "itiephycrenational": "000001",
          "lprenomcrenational": "John", 
          "lnomcrenational": "Doe", 
          "ladrmailcrenational": "johndoe@test.com", 
        },
        "externalData": nil,
        "traceability": {
          "createdBy": "idrex_test_user_user_rw",
          "createdDate": "2020-04-01T12:40:26.196+0000",
          "lastModifiedBy": "idrex_test_user_user_rw",
          "lastModifiedDate": "2020-04-01T12:40:26.196+0000"
        }
      }
    end 
    let!(:user2) { create(:user, id_cristal: '000002') }

    subject do
      client = Hermes::RexClient::Client.new
      Hermes::RexClient::UpdateStructureTask.new.update_entity(rex_entity_json, 'Hermes::Entity', client)
    end 

    describe 'new cre national with new user' do
      it 'works' do 
        subject
        
        user = User.find_by(id_cristal: '000001')
        user_right = Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')
        expect(user_right.user_id).to eq(user.id)
        expect(entity.reload.cre_national_id_cristal).to eq('000001')
        expect(entity.reload.cre_national_email).to eq('johndoe@test.com')
        expect(entity.reload.cre_national_first_name).to eq('John')
        expect(entity.reload.cre_national_last_name).to eq('Doe')
      end 
    end 

    describe 'new cre national with existing user matching with id cristal' do
      let!(:user) { create(:user, email: 'test@olo.com', id_cristal: '000001', first_name: 'John', last_name: 'Doe', password: '12345678', password_confirmation: '12345678')}

      it 'works' do
        subject

        user_right = Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')
        expect(user_right.user_id).to eq(user.id)
        expect(entity.reload.cre_national_id_cristal).to eq('000001')
        expect(entity.reload.cre_national_email).to eq('johndoe@test.com')
        expect(entity.reload.cre_national_first_name).to eq('John')
        expect(entity.reload.cre_national_last_name).to eq('Doe')

      end 
    end 

    describe 'new cre national with existing user matching with email' do
      let!(:user) { create(:user, email: 'johndoe@test.com', first_name: 'John', last_name: 'Doe', password: '12345678', password_confirmation: '12345678')}

      it 'works' do
        subject

        user_right = Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')
        expect(user_right.user_id).to eq(user.id)
        expect(entity.reload.cre_national_id_cristal).to eq('000001')
        expect(entity.reload.cre_national_email).to eq('johndoe@test.com')
        expect(entity.reload.cre_national_first_name).to eq('John')
        expect(entity.reload.cre_national_last_name).to eq('Doe')
        expect(user.reload.id_cristal).to eq('000001')
      end
    end 

    describe 'updating cre national' do
      let!(:user_right) {create(:user_right, user_id: user2.id, entity_id: entity.id, role: 'cre_national', right: 'administrator', type: 'entity')}
      
      it 'works' do
        entity.update(cre_national_id_cristal: '000002', cre_national_email: user2.email)
      
        VCR.use_cassette("rex_cre_national_check_user", :match_requests_on => [:method, :uri, :path]) do
          subject
        end

        user = User.find_by(id_cristal: '000001')
        user_right = Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')
        expect(user_right.user_id).to eq(user.id)
        expect(entity.reload.cre_national_id_cristal).to eq('000001')
        expect(entity.reload.cre_national_email).to eq('johndoe@test.com')
        expect(entity.reload.cre_national_first_name).to eq('John')
        expect(entity.reload.cre_national_last_name).to eq('Doe')
        expect(entity.readable_by?(user)).to eq(true)
        expect(entity.readable_by?(user2)).to eq(false)
      end 
    end 

    describe 'already cre' do
      let!(:user) { create(:user, email: 'johndoe@test.com', id_cristal: '000001', first_name: 'John', last_name: 'Doe', password: '12345678', password_confirmation: '12345678')}
      let!(:user_right_local) { create(:user_right, user_id: user.id, entity_id: entity.id, right: 'administrator', type: 'entity')}
      
      it 'works' do
        subject

        user_right = Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')
        expect(user_right.user_id).to eq(user.id)
        expect(entity.reload.cre_national_id_cristal).to eq('000001')
        expect(entity.reload.cre_national_email).to eq('johndoe@test.com')
        expect(entity.reload.cre_national_first_name).to eq('John')
        expect(entity.reload.cre_national_last_name).to eq('Doe')
        expect(Hermes::UserRight.where(entity_id: entity.id, user: user).count).to eq(1)
      end
    end 

    describe 'remove cre national' do
      let!(:user) { create(:user, id_cristal: '000001')}
      let!(:user_right) {create(:user_right, user_id: user.id, entity_id: entity.id, role: 'cre_national', right: 'administrator', type: 'entity')}
      
      it 'works' do
        entity.update(cre_national_id_cristal: '000001', cre_national_email: user.email)
        rex_entity_json[:fields][:itiephycrenational] = ''

        VCR.use_cassette("rex_cre_national_check_user", :match_requests_on => [:method, :uri, :path]) do
          subject
        end

        expect(Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')).to be_nil
        expect(entity.reload.cre_national_id_cristal).to be_nil
        expect(entity.reload.cre_national_email).to be_nil
        expect(entity.reload.cre_national_first_name).to be_nil
        expect(entity.reload.cre_national_last_name).to be_nil
      end 
    end 

    describe 'removing cre national but still cre' do
      let!(:user_right) {create(:user_right, user_id: user2.id, entity_id: entity.id, role: 'cre_national', right: 'administrator', type: 'entity')}

      it 'works' do
        rex_entity_json[:fields][:itiephycrenational] = ''
        entity.update(cre_national_id_cristal: '000002', cre_national_email: user2.email)

        VCR.use_cassette("rex_cre_national_check_user", :match_requests_on => [:method, :uri, :path]) do
          subject
        end

        user_right = Hermes::UserRight.find_by(entity_id: entity.id)
        expect(user_right.user_id).to eq(user2.id)
        expect(user_right.role).to eq('cre')

        expect(entity.reload.cre_national_id_cristal).to be_nil
        expect(entity.reload.cre_national_email).to be_nil
        expect(entity.reload.cre_national_first_name).to be_nil
        expect(entity.reload.cre_national_last_name).to be_nil
        expect(Hermes::UserRight.where(entity_id: entity.id, user: user2).count).to eq(1)
      end 
    end 

    describe 'removing cre national and missing from REX' do
      let!(:user_right) {create(:user_right, user_id: user2.id, entity_id: entity.id, role: 'cre_national', right: 'administrator', type: 'entity')}

      it 'works' do
        rex_entity_json[:fields][:itiephycrenational] = ''
        entity.update(cre_national_id_cristal: '000003', cre_national_email: user2.email)

        VCR.use_cassette("rex_cre_national_check_user", :match_requests_on => [:method, :uri, :path]) do
          subject
        end

        expect(Hermes::UserRight.find_by(entity_id: entity.id, role: 'cre_national')).to be_nil
        expect(entity.reload.cre_national_id_cristal).to be_nil
        expect(entity.reload.cre_national_email).to be_nil
        expect(entity.reload.cre_national_first_name).to be_nil
        expect(entity.reload.cre_national_last_name).to be_nil
      end 
    end 
  end 
end
